==> v1.0.0.tar.gz: dchat/bot/src/main/java/de/hhhammer/dchat/bot/discord/ServerMessageHandler.java <==
package de.hhhammer.dchat.bot.discord;

import de.hhhammer.dchat.bot.openai.ChatGPTRequestBuilder;
import de.hhhammer.dchat.bot.openai.ChatGPTService;
import de.hhhammer.dchat.bot.openai.ResponseException;
import de.hhhammer.dchat.db.ServerDBService;
import de.hhhammer.dchat.db.models.server.ServerMessage;
import org.javacord.api.entity.DiscordEntity;
import org.javacord.api.entity.message.Message;
import org.javacord.api.entity.message.MessageReference;
import org.javacord.api.entity.message.MessageType;
import org.javacord.api.event.message.MessageCreateEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public class ServerMessageHandler implements MessageHandler {

    private static final Logger logger = LoggerFactory.getLogger(ServerMessageHandler.class);
    private final ServerDBService serverDBService;
    private final ChatGPTService chatGPTService;

    public ServerMessageHandler(ServerDBService serverDBService, ChatGPTService chatGPTService) {
        this.serverDBService = serverDBService;
        this.chatGPTService = chatGPTService;
    }

    @Override
    public void handle(MessageCreateEvent event) throws ResponseException, IOException, InterruptedException {
        String content = extractContent(event);
        var serverId = event.getServer().get().getId();
        var systemMessage = this.serverDBService.getConfig(String.valueOf(serverId)).get().systemMessage();
        var request = event.getMessage().getType() == MessageType.REPLY ?
                new ChatGPTRequestBuilder()
                        .replyRequest(event.getMessage()
                                        .getMessageReference()
                                        .map(MessageReference::getMessage)
                                        .flatMap(m -> m)
                                        .map(Message::getReadableContent)
                                        .stream().toList(),
                                content, systemMessage) :
                new ChatGPTRequestBuilder().simpleRequest(content, systemMessage);
        var response = this.chatGPTService.submit(request);
        if (response.choices().isEmpty()) {
            event.getMessage().reply("No response available");
            return;
        }
        var answer = response.choices().get(0).message().content();
        logServerMessage(event, response.usage().totalTokens());
        event.getMessage().reply(answer);
    }

    @Override
    public boolean isAllowed(MessageCreateEvent event) {
        if (event.getServer().isEmpty()) {
            return false;
        }

        var serverId = event.getServer().get().getId();
        var config = this.serverDBService.getConfig(String.valueOf(serverId));
        if (config.isEmpty()) {
            logger.debug("Not allowed with id: " + serverId);
            return false;
        }
        return true;
    }

    @Override
    public boolean exceedsRate(MessageCreateEvent event) {
        var serverId = String.valueOf(event.getServer().get().getId());
        var config = this.serverDBService.getConfig(serverId);
        if (config.isEmpty()) {
            logger.error("Missing configuration for server with id: " + serverId);
            return true;
        }
        var rateLimit = config.get().rateLimit();
        var countMessagesInLastMinute = this.serverDBService.countMessagesInLastMinute(serverId);

        return countMessagesInLastMinute >= rateLimit;
    }

    @Override
    public boolean canHandle(MessageCreateEvent event) {
        return event.isServerMessage();
    }

    private void logServerMessage(MessageCreateEvent event, int tokens) {
        var serverId = event.getServer().map(DiscordEntity::getId).get();
        var userId = event.getMessageAuthor().getId();

        var serverMessage = new ServerMessage.NewServerMessage(String.valueOf(serverId), userId, tokens);
        this.serverDBService.addMessage(serverMessage);
    }

    private String extractContent(MessageCreateEvent event) {
        long ownId = event.getApi().getYourself().getId();
        return event.getMessageContent().replaceFirst("<" + ownId + "> ", "");
    }
}

==> v1.0.2.tar.gz: dchat/bot/src/main/java/de/hhhammer/dchat/bot/discord/ServerMessageHandler.java <==
package de.hhhammer.dchat.bot.discord;

import de.hhhammer.dchat.bot.openai.ChatGPTRequestBuilder;
import de.hhhammer.dchat.bot.openai.ChatGPTService;
import de.hhhammer.dchat.bot.openai.MessageContext.ReplyInteraction;
import de.hhhammer.dchat.bot.openai.ResponseException;
import de.hhhammer.dchat.db.ServerDBService;
import de.hhhammer.dchat.db.models.server.ServerMessage;
import org.javacord.api.entity.DiscordEntity;
import org.javacord.api.entity.message.Message;
import org.javacord.api.entity.message.MessageReference;
import org.javacord.api.entity.message.MessageType;
import org.javacord.api.event.message.MessageCreateEvent;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.List;

public class ServerMessageHandler implements MessageHandler {

    private static final Logger logger = LoggerFactory.getLogger(ServerMessageHandler.class);
    private final ServerDBService serverDBService;
    private final ChatGPTService chatGPTService;

    public ServerMessageHandler(ServerDBService serverDBService, ChatGPTService chatGPTService) {
        this.serverDBService = serverDBService;
        this.chatGPTService = chatGPTService;
    }

    @Override
    public void handle(MessageCreateEvent event) throws ResponseException, IOException, InterruptedException {
        String content = extractContent(event);
        var serverId = event.getServer().get().getId();
        var systemMessage = this.serverDBService.getConfig(String.valueOf(serverId)).get().systemMessage();
        List<ReplyInteraction> messageContext = event.getMessage().getType() == MessageType.REPLY ? getContextMessages(event) : List.of();
        var request = new ChatGPTRequestBuilder().contextRequest(messageContext, content, systemMessage);
        var response = this.chatGPTService.submit(request);
        if (response.choices().isEmpty()) {
            event.getMessage().reply("No response available");
            return;
        }
        var answer = response.choices().get(0).message().content();
        logServerMessage(event, response.usage().totalTokens());
        event.getMessage().reply(answer);
    }

    @Override
    public boolean isAllowed(MessageCreateEvent event) {
        if (event.getServer().isEmpty()) {
            return false;
        }

        var serverId = event.getServer().get().getId();
        var config = this.serverDBService.getConfig(String.valueOf(serverId));
        if (config.isEmpty()) {
            logger.debug("Not allowed with id: " + serverId);
            return false;
        }
        return true;
    }

    @Override
    public boolean exceedsRate(MessageCreateEvent event) {
        var serverId = String.valueOf(event.getServer().get().getId());
        var config = this.serverDBService.getConfig(serverId);
        if (config.isEmpty()) {
            logger.error("Missing configuration for server with id: " + serverId);
            return true;
        }
        var rateLimit = config.get().rateLimit();
        var countMessagesInLastMinute = this.serverDBService.countMessagesInLastMinute(serverId);

        return countMessagesInLastMinute >= rateLimit;
    }

    @Override
    public boolean canHandle(MessageCreateEvent event) {
        return event.isServerMessage();
    }

    private void logServerMessage(MessageCreateEvent event, int tokens) {
        var serverId = event.getServer().map(DiscordEntity::getId).get();
        var userId = event.getMessageAuthor().getId();

        var serverMessage = new ServerMessage.NewServerMessage(String.valueOf(serverId), userId, tokens);
        this.serverDBService.addMessage(serverMessage);
    }

    private String extractContent(MessageCreateEvent event) {
        long ownId = event.getApi().getYourself().getId();
        return event.getMessageContent().replaceFirst("<" + ownId + "> ", "");
    }

    @NotNull
    private List<ReplyInteraction> getContextMessages(MessageCreateEvent event) {
        return event.getMessage()
                .getMessageReference()
                .map(MessageReference::getMessage)
                .flatMap(m -> m)
                .map(Message::getReadableContent)
                .map(ReplyInteraction::new)
                .stream()
                .toList();
    }
}
